<?php

namespace SRVX\Service\Pay;

use SRVX\Service\BaseService;
use AlipaySdk\Request\AlipayFundTransPagePayRequest;


class ExtraAlipayService extends BaseService
{

    private $config = [];
    private $aopClient;

    public $gatewayUrl = 'https://openapi.alipay.com/gateway.do';
    public $apiVersion = '1.0';
    public $signType = 'RSA2';
    public $postCharset = 'UTF-8';
    public $format = 'json';

    public function __construct($config = [])
    {
        $this->getConfig($config);
        $this->aopClient();
    }


    private function aopClient()
    {
        $this->aopClient        = new \AopClient();
        $this->aopClient->appId = $this->config['pay_ali_app_id'];
        //支付宝私钥值
        $this->aopClient->rsaPrivateKey = $this->config['pay_ali_private_key'];
        //支付宝公钥值
        $this->aopClient->alipayrsaPublicKey = $this->config['pay_ali_public_key'];

        $this->aopClient->gatewayUrl  = $this->gatewayUrl;
        $this->aopClient->apiVersion  = $this->apiVersion;
        $this->aopClient->signType    = $this->signType;
        $this->aopClient->postCharset = $this->postCharset;
        $this->aopClient->format      = $this->format;
    }


    private function getConfig($config)
    {
        if (empty($this->config)) {
            $config       = $this->s->GeneralSet()->getSet(['pay_ali_app_id', 'pay_ali_private_key', 'pay_ali_public_key'], false);
            $this->config = $config;
        }
        $this->config = $config;
    }

    /**
     * alipay.fund.trans.page.pay（pc端 - 批次支付接口）
     * @param $order_id  string 支付宝批次订单号。(alipay.fund.batch.create接口返回的batch_trans_id的值）
     * @param $app_auth_token string app_auth_token，完成ISV代调用
     * @param $return_url string return_url 支付完成同步回调地址
     * @param $notify_url string notify_url，支付完成异步回调地址
     * @param string $product_code 销售产品码 示例值：BATCH_API_TO_ACC
     * @param string $biz_scene 业务场景 示例值：MESSAGE_BATCH_PAY
     * @return array
     * @throws \Exception
     */
    public function tradeFundTransPagePay($order_id, $app_auth_token, $return_url, $notify_url, $passback_params = "", $product_code = "BATCH_API_TO_ACC", $biz_scene = "MESSAGE_BATCH_PAY")
    {
        $request   = new \AlipayFundTransPagePayRequest();
        $bizParams = [
            "order_id"        => strval($order_id),
            "product_code"    => $product_code,
            "biz_scene"       => $biz_scene,
            "passback_params" => $passback_params
        ];
        $request->setReturnUrl($return_url);
        $request->setNotifyUrl($notify_url);
        $request->setBizContent(json_encode($bizParams));
        return $this->aopClient->pageExecute($request, "GET", $app_auth_token);
    }
}