<?php

namespace SRVX\Utils;
/**
 * Class Curl
 * @package SRVX\Utils
 */
class Curl
{
	/**
	 * @param string $url
	 * @param bool $ssl
	 * @param int $timeout
	 * @return bool|string
	 */
	public static function Get(string $url, bool $ssl = false, int $timeout = 30)
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $ssl); // 让CURL支持HTTPS访问
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_TIMEOUT, $timeout);
		$res = curl_exec($ch);
		curl_close($ch);
		return $res;
	}

	/**
	 * @param string $url
	 * @param array $params
	 * @param bool $ssl
	 * @param bool $use_http_build_query
	 * @param int $timeout
	 * @param bool $use_json_query
	 * @param array $http_header
	 * @return bool|string
	 */
	public function Post(string $url, array $params, bool $ssl = false, bool $use_http_build_query = true, int $timeout = 30, bool $use_json_query = false,array $http_header)
	{
		$use_http_build_query && $params = http_build_query($params);
		$use_json_query && $params = json_encode($params);
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $ssl); // 让CURL支持HTTPS访问
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
		curl_setopt($ch, CURLOPT_TIMEOUT, 30);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $params);
		if($use_json_query)
		{
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");
			$http_header && curl_setopt($ch, CURLOPT_HTTPHEADER, $http_header);
		}
		$res = curl_exec($ch);
		curl_close($ch);
		return $res;
	}

    public function isSiteAvailable($url){
        // Check, if a valid url is provided
        /*if(!filter_var($url, FILTER_VALIDATE_URL)){
            return false;
        }*/

        // Initialize cURL
        $curlInit = curl_init($url);

        // Set options
        curl_setopt($curlInit,CURLOPT_CONNECTTIMEOUT,10);
        curl_setopt($curlInit,CURLOPT_HEADER,true);
        curl_setopt($curlInit,CURLOPT_NOBODY,true);
        curl_setopt($curlInit,CURLOPT_RETURNTRANSFER,true);

        curl_exec($curlInit);
        $httpCode = curl_getinfo($curlInit, CURLINFO_HTTP_CODE);

        // Close a cURL session
        curl_close($curlInit);

        return $httpCode;
    }
}